@@ -121,6 +121,12 @@ Atom net_wm_window_type_utility;
 Atom net_workarea;
 Atom utf8_string;
 
+/* Systray similation for older KDE apps */
+Atom kde_net_wm_system_tray_window_for;
+Atom net_system_tray_manager;
+Atom net_system_tray_selection;
+Atom net_system_tray_opcode;
+
 static gboolean
 check_type_and_format (Display * dpy, Window w, Atom a, int expected_format,
     Atom expected_type, int n_items, int format, Atom type)
@@ -291,6 +297,21 @@ initGnomeHints (Display * dpy)
     win_workspace = XInternAtom (dpy, "_WIN_WORKSPACE", FALSE);
 }
 
+void
+initSystrayHints (Display * dpy, int nscreen)
+{
+    gchar selection[32];
+
+    TRACE ("entering initSystrayHints");
+
+    g_snprintf (selection, sizeof (selection), "_NET_SYSTEM_TRAY_S%d", nscreen);
+    net_system_tray_manager   = XInternAtom (dpy, "MANAGER", FALSE);
+    net_system_tray_opcode    = XInternAtom (dpy, "_NET_SYSTEM_TRAY_OPCODE", FALSE);
+    net_system_tray_selection = XInternAtom (dpy, selection, FALSE);
+    kde_net_wm_system_tray_window_for = 
+        XInternAtom (dpy, "_KDE_NET_WM_SYSTEM_TRAY_WINDOW_FOR", FALSE);
+}
+
 gboolean
 getHint (Display * dpy, Window w, Atom a, long *value)
 {
@@ -710,7 +731,11 @@ get_utf8_string (Display * dpy, Window w, Atom xatom, char **str_p)
         return FALSE;
     }
 
-    utf8_string_remove_controls((gchar *) str, -1, NULL);
+    if (str)
+    {
+        utf8_string_remove_controls((gchar *) str, -1, NULL);
+    }
+    
     *str_p = str;
 
     return TRUE;
@@ -752,6 +777,10 @@ get_text_property (Display * dpy, Window w, Atom a)
     if (XGetTextProperty (dpy, w, &text, a))
     {
         retval = text_property_to_utf8 (dpy, &text);
+        if (retval)
+        {
+            utf8_string_remove_controls((gchar *) retval, -1, NULL);
+        }
         if ((text.value) && (text.nitems > 0))
         {
             XFree (text.value);
@@ -766,7 +795,6 @@ get_text_property (Display * dpy, Window w, Atom a)
     return retval;
 }
 
-
 void
 getWindowName (Display * dpy, Window w, char **name)
 {
@@ -796,6 +824,69 @@ getWindowName (Display * dpy, Window w, char **name)
     }
 }
 
+gboolean
+checkKdeSystrayWindow(Display * dpy, Window window)
+{
+    Atom actual_type;
+    int actual_format;
+    unsigned long nitems;
+    unsigned long bytes_after;
+    Window trayIconForWindow;
+
+    TRACE ("entering GetWindowRole");
+    g_return_val_if_fail (window != None, FALSE);
+    
+    XGetWindowProperty(dpy, window, kde_net_wm_system_tray_window_for, 0L, 
+            sizeof(Window), FALSE, XA_WINDOW, &actual_type, &actual_format, 
+            &nitems, &bytes_after, (unsigned char **)&trayIconForWindow);
+
+    if ((actual_format == None) || 
+        (actual_type != XA_WINDOW) || 
+        (trayIconForWindow == None))
+    {
+        return FALSE;
+    }
+    return TRUE;
+}
+
+void
+sendSystrayReqDock(Display * dpy, Window window, Window systray)
+{
+    XClientMessageEvent xev;
+
+    TRACE ("entering sendSystrayReqDock");
+    g_return_if_fail (window != None);
+    g_return_if_fail (systray != None);
+
+    xev.type = ClientMessage;
+    xev.window = systray;
+    xev.message_type = net_system_tray_opcode;
+    xev.format = 32;
+    xev.data.l[0] = CurrentTime;
+    xev.data.l[1] = 0; /* SYSTEM_TRAY_REQUEST_DOCK */
+    xev.data.l[2] = window;
+    xev.data.l[3] = 0; /* Nada */
+    xev.data.l[4] = 0; /* Niet */
+
+    XSendEvent (dpy, systray, FALSE, NoEventMask, (XEvent *) & xev);
+}
+
+Window
+getSystrayWindow (Display * dpy)
+{
+    Window systray_win = None;
+
+    TRACE ("entering getSystrayWindow");
+
+    systray_win = XGetSelectionOwner (dpy, net_system_tray_selection);
+    if (systray_win)
+    {
+        XSelectInput (dpy, systray_win, StructureNotifyMask);
+    }
+    TRACE ("New systray window:  0x%lx", systray_win);
+    return systray_win;
+}
+
 gboolean
 getWindowRole (Display * dpy, Window window, char **role)
 {
