@@ -47,7 +47,6 @@ workspaceSwitch (int new_ws, Client * c2)
     int rx, ry, wx, wy;
     unsigned int mask;
     unsigned long data[1];
-    XEvent an_event;
 
     TRACE ("entering workspaceSwitch");
 
@@ -65,6 +64,10 @@ workspaceSwitch (int new_ws, Client * c2)
         return;
     }
 
+    /* Grab the pointer to avoid side effects with EnterNotify events */
+    XGrabPointer (dpy, gnome_win, FALSE, EnterWindowMask, GrabModeAsync,
+                       GrabModeAsync, None, None, CurrentTime);
+
     workspace = new_ws;
     if (c2)
     {
@@ -84,7 +87,7 @@ workspaceSwitch (int new_ws, Client * c2)
             if (c == previous)
             {
                 CLIENT_FLAG_SET (previous, CLIENT_FLAG_FOCUS);
-                clientSetFocus (NULL, FALSE, TRUE);
+                clientSetFocus (NULL, FOCUS_IGNORE_MODAL);
             }
             if (!clientIsTransientOrModal (c))
             {
@@ -148,15 +151,13 @@ workspaceSwitch (int new_ws, Client * c2)
     XChangeProperty (dpy, root, net_current_desktop, XA_CARDINAL, 32,
         PropModeReplace, (unsigned char *) data, 1);
     workspaceUpdateArea (margins, gnome_margins);
-    XSync (dpy, FALSE);
+
+    /* Ungrab the pointer we grabbed before mapping/unmapping all windows */
+    XUngrabPointer (dpy, CurrentTime);
+
     if (!(params.click_to_focus))
     {
-        /* Just get rid of EnterNotify events when using focus follow mouse */
-        while (XCheckTypedEvent (dpy, EnterNotify, &an_event))
-            ; /* VOID */
-        if (!(c2)
-            && (XQueryPointer (dpy, root, &dr, &window, &rx, &ry, &wx, &wy,
-                    &mask)))
+        if (!(c2) && (XQueryPointer (dpy, root, &dr, &window, &rx, &ry, &wx, &wy, &mask)))
         {
             c = clientAtPosition (rx, ry, NULL);
             if (c)
@@ -165,7 +166,7 @@ workspaceSwitch (int new_ws, Client * c2)
             }
         }
     }
-    clientSetFocus (new_focus, TRUE, FALSE);
+    clientSetFocus (new_focus, NO_FOCUS_FLAG);
 }
 
 void
